// ----------------------------------------------------------------------------
//
// rt_block.h
// RT++ heap block layout
//
// $Id: rt_block.h,v 1.8 1996/04/04 07:05:39 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_BLOCK_H
#define RT_BLOCK_H

// ----------------------------------------------------------------------------
// 
// header information of each heap block
// the marking scheme assumes that heap blocks start at even addresses
//
// ----------------------------------------------------------------------------
class Block
{
  public:
  int size;                     // size of block in bytes
  int mark(void);               // mark block; return 1 if already marked
  int unmark(void);             // unmark block; return 0 if it was not marked
  int is_marked(void);          // true iff block was marked
  static int alignment(void);   // return alignment
  static int align(int);        // return aligned memory block size
  static Block *block(void *);  // return block info from memory address
};

// ----------------------------------------------------------------------------
//
// free (unused) heap block
// in our scheme all blocks have minimum size of free blocks
// and are aligned to free blocks
// we assume that sizeof(FreeBlock) is a power of two,
// if not, redefine Block::alignment()!!!
//
// ----------------------------------------------------------------------------
class FreeBlock: public Block
{
  public:
  FreeBlock *next;              // link to next free block
};

#endif
// ----------------------------------------------------------------------------
// end of rt_block.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
