// ----------------------------------------------------------------------------
//
// rt_list.h
// RT++ garbage collected lists
//
// $Id: rt_list.h,v 1.13 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_LIST_H
#define RT_LIST_H

// ----------------------------------------------------------------------------
//
// a list node with element of type <T>
// (gcc does not accept this as a non-template local class in <List>)
//
// ----------------------------------------------------------------------------

template <class T>
class ListNode
{
  public:
  T head;            // the head element
  ListNode<T> *tail; // pointer to next node
};

// ----------------------------------------------------------------------------
//
// pointer to a list node with element of type <T>
//
// ----------------------------------------------------------------------------
template <class T>
class List
{
  friend class ThreadBagCore;
  protected:
  ListNode<T> *pointer;                             // pointer to first node
  static Block *block(ListNode<T> *p);              // block info from pointer
  List(ListNode<T> *p);                             // core ptr duplication
  void _cons(const T &_head, const List<T> &_tail); // core node allocation
  public:
  void init(void);                                  // initialize nil list
  void nil(void);                                   // initialize nil list
  void cons(const T &_head, const List<T> &_tail);  // allocate list node
  List(void);                                       // construct nil list
  List(const T &_head, const List<T> &_tail);       // construct list node
  List(const List<T> &l);                           // duplicate list reference
  ~List(void);                                      // destroy list reference
  List<T>& operator = (const List<T> &l);           // assign list references
  int operator == (const List<T> &l);               // compare list references
  friend int Null(const List<T> &l);                // is list nil?
  static List<T> Nil(void);                         // nil list
  friend List<T> Cons(const T &_head, const List<T> &_tail); // construct node
  friend T Head(const List<T> &l);                  // head element of list
  void head(const T &_head);                        // set head element
  friend List<T> Tail(const List<T> &l);            // tail element of list
  void tail(const List<T> &_tail);                  // set tail element
  void tail(void);                                  // set reference to tail
  void scan (void);                                 // scan function for type
};

#endif
// ----------------------------------------------------------------------------
// end of rt_list.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
