// ----------------------------------------------------------------------------
//
// rt_mutex.i
// RT++ mutual exclusion objects
//
// $Id: rt_mutex.i,v 1.4 1996/03/14 10:41:25 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_MUTEX_I
#define RT_MUTEX_I

// ----------------------------------------------------------------------------
//
// initialize mutex
//
// ----------------------------------------------------------------------------
inline void
Mutex::init(void) 
{
#ifdef RT_PARISC
  unsigned long base = (unsigned long)cache;
  unsigned long align =  16;
  mutex = (int*)(base+(align-base%align)%align);
  *mutex = 1;
#else
  mutex = 0;
#endif
}

// ----------------------------------------------------------------------------
//
// create mutex
//
// ----------------------------------------------------------------------------
inline
Mutex::Mutex(void) 
{
  init();
}

// ----------------------------------------------------------------------------
//
// lock mutex
//
// ----------------------------------------------------------------------------
inline void
Mutex::lock(void)
{
#ifdef RT_PARISC
  while (testAndSet(mutex) == 0)  // test and set lock
    { 
      while (*mutex == 0) { }     // read lock to reduce bus traffic
    }
#else
  while (testAndSet(&mutex) == 1) // test and set lock
    { 
      while (mutex == 1) { }      // read lock to reduce bus traffic
    }
#endif
}

// ----------------------------------------------------------------------------
//
// unlock mutex
//
// ----------------------------------------------------------------------------
inline void 
Mutex::unlock(void)
{
#ifdef RT_PARISC
  *mutex = 1;
#else
  mutex = 0;
#endif
}

// ----------------------------------------------------------------------------
//
// lock mutex but return 0 if not successful
//
// ----------------------------------------------------------------------------
inline int
Mutex::test(void)
{
#ifdef RT_PARISC
  return (testAndSet(mutex) == 1);
#else
  return (testAndSet(&mutex) == 0);
#endif
}

#endif
// ----------------------------------------------------------------------------
// end of rt_mutex.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
