// ----------------------------------------------------------------------------
//
// rt_pointer.h
// RT++ garbage collected pointer objects
//
// $Id: rt_pointer.h,v 1.13 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_POINTER_H
#define RT_POINTER_H

// ----------------------------------------------------------------------------
//
// pointer to object of type <T>
//
// ----------------------------------------------------------------------------
template <class T>
class Pointer
{
  protected:
  T *pointer;                                   // the memory location
  static Block *block(T *p);                    // block info from location
  void _alloc(void);                            // core allocation
  public:
  void init(void);                              // initialize nil pointer
  void alloc(void);                             // allocate object
  Pointer(void);                                // construct nil pointer
  Pointer(int n);                               // construct object (n==1)
  Pointer(const Pointer<T> &p);                 // duplicate pointer
  ~Pointer(void);                               // destroy pointer
  Pointer<T>& operator = (const Pointer<T> &p); // assign pointers
  int operator == (const Pointer<T> &p);        // compare pointers
  friend int Null(const Pointer<T> &p);         // is pointer nil?
  static Pointer<T> Alloc(void);                // allocate object
  T& operator * (void);                         // dereference pointer
  T* operator -> (void);                        // dereference pointer
  friend ostream& operator << (ostream& s, const Pointer<T>& p); // output ptr
  void scan(void);                              // scan function for this type
};

#endif
// ----------------------------------------------------------------------------
// end of rt_pointer.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
