// ----------------------------------------------------------------------------
//
// rt_proc.h
// RT++ processor-global data
//
// $Id: rt_proc.h,v 1.22 1996/02/29 11:51:13 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_PROC_H
#define RT_PROC_H

// ----------------------------------------------------------------------------
//
// global data for current processor
// there is only one object of this type per process!!!
// all class variables may be allocated in static area
// all class functions may be declared static
//
// ----------------------------------------------------------------------------
class Processor
{
  static int kernel;                      // set if thread is in kernel mode

  static CoreThread *thread;              // currently executed thread
  static Stack shadow;                    // current shadow stack
  static CoreThread *sched;               // scheduler thread

  static int num;                         // number of processor
  static FreeBlock *free;                 // current free memory block
  static FreeBlock *prev;                 // predecessor in free list
  static int size;                        // size of current free block

  static sigset_t empty_mask;             // empty signal mask
  static sigset_t alarm_mask;             // mask with SIGALRM
  static sigset_t gc_mask;                // mask with SIGALRM and SIGUSR1

  static int gc_signal;                   // garbage collection signal

  static Block *unlink(int s);            // unlink block from free list
  static Block *search(int s);            // search for block in free list

  public:
  static void init(int n);                // initialize processor
  static int number(void);                // return processor number
  static void save(void);                 // executed before thread switch

  static CoreThread *get_thread(void);    // get current thread
  static void set_thread(CoreThread *);   // set current thread
  static CoreThread *get_sched(void);     // get scheduler thread

  static void lock(void);                 // lock processor
  static void unlock(void);               // unlock processor
  static int locked(void);                // is processor in kernel mode?
  static int in_scheduler(void);          // processor executes sched thread

  static void nopreempt(void);            // switch off alarm preemption
  static void preempt(void);              // switch on alarm preemption
  static void gc_off(void);               // switch off gc preemption
  static void gc_on(void);                // switch on gc preemption

  static void scheduler(void);            // execute scheduler
  static void block(Mutex *);             // block current thread
  static void poll(void);                 // poll for lazy threads

  static void push(void *addr, void(*f)(void*)); // push address on stack
  static void pop(void *addr);            // pop address from stack
  static void gc(void);                   // prepare for gc
  static void collect(void);              // perform gc
  static void *alloc(int s);              // allocate heap memory
};

// processor-object, statically allocated in address space of current process
extern Processor processor;               

#endif
// ----------------------------------------------------------------------------
// end of rt_proc.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
