// ----------------------------------------------------------------------------
//
// rt_proc.i
// RT++ processor-global data
//
// $Id: rt_proc.i,v 1.2 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_PROC_I
#define RT_PROC_I

// ----------------------------------------------------------------------------
//
// get processor number
//
// ----------------------------------------------------------------------------
inline int
Processor::number(void)
{
  return(num);
}

// ----------------------------------------------------------------------------
//
// get current thread
//
// ----------------------------------------------------------------------------
inline CoreThread*
Processor::get_thread(void)
{
  return(thread);
}

// ----------------------------------------------------------------------------
//
// set current thread
//
// ----------------------------------------------------------------------------
inline void
Processor::set_thread(CoreThread *t)
{
  thread = t;
  shadow = *(t->get_shadow());
}

// ----------------------------------------------------------------------------
//
// get scheduler thread
//
// ----------------------------------------------------------------------------
inline CoreThread*
Processor::get_sched(void)
{
  return(sched);
}

// ----------------------------------------------------------------------------
//
// going to switch thread
//
// ----------------------------------------------------------------------------
inline void
Processor::save(void)
{
  thread->set_shadow(&shadow);
}

// ----------------------------------------------------------------------------
//
// prevent preemption of current task
//
// ----------------------------------------------------------------------------
inline void
Processor::lock(void)
{
  kernel = 1;
}

// ----------------------------------------------------------------------------
//
// allow preemption of current task
//
// ----------------------------------------------------------------------------
inline void
Processor::unlock(void)
{
  kernel = 0;
}

// ----------------------------------------------------------------------------
//
// is processor locked?
//
// ----------------------------------------------------------------------------
inline int
Processor::locked(void)
{
  return(kernel);
}

// ----------------------------------------------------------------------------
//
// does processor execute scheduler thread?
//
// ----------------------------------------------------------------------------
inline int
Processor::in_scheduler(void)
{
  return(thread == sched);
}

// ----------------------------------------------------------------------------
//
// push address on shadow stack
//
// ----------------------------------------------------------------------------
inline void
Processor::push(void *addr, void(*fun)(void*))
{
  shadow.push(addr, fun);
}

// ----------------------------------------------------------------------------
//
// pop address from shadow stack
//
// ----------------------------------------------------------------------------
inline void
Processor::pop(void *addr)
{
  shadow.pop(addr);
}

#endif
// ----------------------------------------------------------------------------
// end of rt_proc.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
