// ----------------------------------------------------------------------------
//
// rt_system.h
// RT++ global system data
//
// $Id: rt_system.h,v 1.27 1996/04/04 07:06:03 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_SYSTEM_H
#define RT_SYSTEM_H

// ----------------------------------------------------------------------------
//
// all data global to the system (to be allocated in shared memory)
// there is only one object of this type!!!
//
// ----------------------------------------------------------------------------
class System                    
{
  CoreThread *ready_hd;   // queue of threads ready for execution
  CoreThread *ready_tl;   // tail of queue
  CoreThread *lazy;       // virtual threads, waiting for real tasks to execute
  CoreThread **bound_hd;  // queues of threads bound to specific processors
  CoreThread **bound_tl;  // tails of queues
  CoreThread *root;       // root threads for garbage collection

  Mutex mutex_ready;      // mutex for real threads
  Mutex mutex_lazy;       // mutex for lazy threads
  Mutex *mutex_bound;     // mutexes for bound threads
  Mutex mutex_term;       // mutex for termination
  Mutex mutex_gc;         // mutex for garbage collection
  Mutex mutex_stacks;     // mutex for stack allocation
  Mutex mutex_root;       // mutex for root threads

  int size;               // default stack size of threads
  int procnumber;         // number of Unix processes
  volatile int regnumber; // number of registered processes
  pid_t *procid;          // array of Unix process identifiers
  int code;               // exit code
  int sigval;             // signal value
  void *stacks;           // disposed thread stacks
  int pollcnt;            // count for scheduler polling loop

  void is_bound(CoreThread *next);   // place thread into bound queue
  CoreThread *get_bound(void);       // get thread from bound queue of proc

  public:
  System(int, int);                  // initialize system object
  int procnum(void);                 // return processor number
  int stacksize(void);               // return default stacksize
  void proc(int, pid_t);             // register processor

  void is_ready(CoreThread *next);   // place thread into ready queue
  CoreThread *get_ready(void);       // get thread from ready queue

  void is_lazy(CoreThread *next);    // place thread into lazy queue
  CoreThread *get_lazy(void);        // get thread from lazy queue
  void get_lazy(CoreThread *thread); // get denoted thread from lazy queue

  void lock(void);                   // lock system for termination
  void wait(void);                   // global synchronization on startup
  void shutdown(int code);           // trigger program shutdown 
  int terminate(void);               // terminate children processes

  int ret_code(void);                // return code

  int thread_avail(void);            // return true iff thread available
  void root_thread(CoreThread*);     // add thread to root list
  void no_root_thread(CoreThread*);  // remove thread from root list
  CoreThread *root_threads(void);    // root of garbage collection
  void gc_threads(void);             // remove unmarked threads

  int poll(void);                    // return iteration count for polling
  void gc(void);                     // trigger garbage collection
  void set(int);                     // set signal value
  int get(void);                     // get signal value

  void *stack(int size);             // return appropriate thread stack
  void dispose(void *stack);         // dispose stack for later reuse
};

// system-global data, dynamically allocated in shared memory
extern System *sys; 

// allocate system object and initialize it with global information
void sys_init(int procnum, int stacksize);

#endif
// ----------------------------------------------------------------------------
// end of rt_system.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
