// ----------------------------------------------------------------------------
//
// rt_thread.h
// RT++ interface to system threads
//
// $Id: rt_thread.h,v 1.22 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_THREAD_H
#define RT_THREAD_H

// ----------------------------------------------------------------------------
//
// interface to system threads
//
// ----------------------------------------------------------------------------
class SystemThread 
{
#ifdef RT_QT
  private:
  int size;                       // size of allocated stack    
  qt_t *thread;                   // stack pointer of quick thread
  public:
  void *stack;                    // allocated stack (public for deallocation)
  void save(qt_t *);              // save stack pointer
#endif

  public:
  void init(void);                // init function for this type

  // create thread without stack (virtual)
  SystemThread(void);             

  // realize thread i.e. allocate stack of size <s>
  // and start execution of <shell>(<t>) where <t> points to the thread
  // return the address of the stack
  void * realize(int s, void(*shell)(void*, void*, void(*)(void*)));

  // destroy real thread i.e. deallocate its stack
  void destroy(void);

  void exec_basic(void);          // switch from scheduler to user thread
  void exec_ready(void);          // switch and place thread into ready queue
  void exec_block(Mutex *mutex);  // switch and unlock mutex
  void exec_term(void);           // switch and terminate thread
};

#endif
// ----------------------------------------------------------------------------
// end of rt_thread.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
