// ----------------------------------------------------------------------------
//
// rt_thread.i
// RT++ interface to system threads
//
// $Id: rt_thread.i,v 1.5 1996/03/18 09:39:51 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_THREAD_I
#define RT_THREAD_I

// ----------------------------------------------------------------------------
//
// initialize a new system thread (virtual, without stack)
//
// ----------------------------------------------------------------------------
inline void
SystemThread::init(void)
{
#ifdef RT_QT
  stack = 0;
  size = 0;
  thread = 0;
#endif
}

// ----------------------------------------------------------------------------
//
// create a new system thread (virtual, without stack)
//
// ----------------------------------------------------------------------------
inline
SystemThread::SystemThread(void)
{
  init();
}

#ifdef RT_QT
// ----------------------------------------------------------------------------
//
// align stack as required for quick threads
//
// ----------------------------------------------------------------------------
static inline void*
alignstack(void *s) 
{
  static const int align = QT_STKALIGN-1;
  return((void *)((((qt_word_t)(s)) + align) & ~align));
}
#endif

// ----------------------------------------------------------------------------
//
// realize new thread with stack of size <s> ;
// the new thread will execute <shell(this, 0, 0)> 
// which in turn will execute <this->user()>
// the function returns the address for the shadow stack (0 if none)
//
// ----------------------------------------------------------------------------
inline void *
SystemThread::realize(int s, void(*shell)(void*, void*, void(*)(void*)))
{
#ifdef RT_QT
  size = s;                         // size of stack
  extern void *newstack(int s);     // defined in rt_sys.cc
  stack = newstack(s);              // stack allocation
  thread = QT_ARGS(QT_SP(alignstack(stack),
			 size-QT_STKALIGN), this, 0, 0, shell);
#ifdef RT_GROW_UP
  return(stack+size-QT_STKALIGN);
#else
  return(stack);
#endif
#else
#error thread initialization not implemented
#endif
}

// ----------------------------------------------------------------------------
//
// destroy real thread i.e. deallocate its stack
//
// ----------------------------------------------------------------------------
inline void
SystemThread::destroy(void)
{
  sys->dispose(stack);
}

// ----------------------------------------------------------------------------
//
// switch from old thread to new thread
//
// ----------------------------------------------------------------------------
inline void 
SystemThread::exec_basic(void)
{
#ifdef RT_QT
  // saves <sptr> in <newthread>
  void *switch_basic(qt_t *sptr, void* newthread, void *dummy);
  QT_BLOCK(switch_basic, this, 0, thread);
#else
#error thread switch not implemented
#endif
}

// ----------------------------------------------------------------------------
//
// switch from old thread to new thread; old thread is put into ready queue
//
// ----------------------------------------------------------------------------
inline void 
SystemThread::exec_ready(void)
{
#ifdef RT_QT
  // saves <sptr> in <newthread> and places current thrad into ready queue
  void *switch_ready(qt_t *sptr, void* newthread, void *dummy);
  QT_BLOCK(switch_ready, this, 0, thread);
#else
#error thread switch not implemented
#endif
}

// ----------------------------------------------------------------------------
//
// switch from old to new thread; old thread is blocked and mutex is unlocked
//
// ----------------------------------------------------------------------------
inline void 
SystemThread::exec_block(Mutex *mutex)
{
#ifdef RT_QT
  // saves <sptr> in <newthread> and unlocks blocker task
  void *switch_block(qt_t *sptr, void* newthread, void *blocker);
  QT_BLOCK(switch_block, this, mutex, thread);
#else
#error thread switch not implemented
#endif
}

// ----------------------------------------------------------------------------
//
// switch from old thread to new thread; old thread is terminated
//
// ----------------------------------------------------------------------------
inline void 
SystemThread::exec_term(void)
{
#ifdef RT_QT
  // saves <sptr> in <newthread> and unlocks blocker task
  void *switch_term(qt_t *sptr, void* newthread, void *blocker);
  QT_BLOCK(switch_term, this, 0, thread);
#else
#error thread switch not implemented
#endif
}

#endif
// ----------------------------------------------------------------------------
// end of rt_thread.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
