// ----------------------------------------------------------------------------
//
// rt_tptr.h
// RT++ pointer to threads (garbage collected)
//
// $Id: rt_tptr.h,v 1.10 1996/03/21 12:28:55 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_TPTR_H
#define RT_TPTR_H

// ----------------------------------------------------------------------------
//
// thread pointer typed in result only
//
// ----------------------------------------------------------------------------
template <class R>
class Thread: public Pointer< ResultThread<R> >
{
  public:
  Thread(void);
  Thread(ResultThread<R> *p);
  friend R Wait(const Thread<R> &t);
};

// ----------------------------------------------------------------------------
//
// thread pointer with no argument
//
// ----------------------------------------------------------------------------
template <class R>
class Thread0: public Pointer< ArgumentThread0<R> >
{
  void _init(R (*f)(void));
  void _create(R (*f)(void));
  void _start(R (*f)(void));
  public:
  void create(R (*f)(void));
  void start(R (*f)(void));
  friend Thread0<R> Create(R (*f)(void));
  friend Thread0<R> Start(R (*f)(void));  
  Thread0(void);
  Thread0(R (*f)(void));
  friend R Wait(const Thread0<R> &t);
  operator Thread<R>();
};

// ----------------------------------------------------------------------------
//
// thread pointer with one argument
//
// ----------------------------------------------------------------------------
template <class R, class A1>
class Thread1: public Pointer< ArgumentThread1<R, A1> >
{
  void _init(R (*f)(A1), A1 a1);
  void _create(R (*f)(A1), A1 a1);
  void _start(R (*f)(A1), A1 a1);
  public:
  void create(R (*f)(A1), A1 a1);
  void start(R (*f)(A1), A1 a1);
  friend Thread1<R, A1> Create(R (*f)(A1), A1 a1);
  friend Thread1<R, A1> Start(R (*f)(A1), A1 a1);
  Thread1(void);
  Thread1(R (*f)(A1), A1 a1);
  friend R Wait(const Thread1<R, A1> &t);
  operator Thread<R>();
};

// ----------------------------------------------------------------------------
//
// thread pointer with two arguments
//
// ----------------------------------------------------------------------------
template <class R, class A1, class A2>
class Thread2: public Pointer< ArgumentThread2<R, A1, A2> >
{
  void _init(R (*f)(A1, A2), A1 a1, A2 a2);
  void _create(R (*f)(A1, A2), A1 a1, A2 a2);
  void _start(R (*f)(A1, A2), A1 a1, A2 a2);
  public:
  void create(R (*f)(A1, A2), A1 a1, A2 a2);
  void start(R (*f)(A1, A2), A1 a1, A2 a2);
  friend Thread2<R, A1, A2> Create(R (*f)(A1, A2), A1 a1, A2 a2);
  friend Thread2<R, A1, A2> Start(R (*f)(A1, A2), A1 a1, A2 a2);
  Thread2(void);
  Thread2(R (*f)(A1, A2), A1 a1, A2 a2);
  friend R Wait(const Thread2<R, A1, A2> &t);
  operator Thread<R>();
};

// ----------------------------------------------------------------------------
//
// thread pointer with three arguments
//
// ----------------------------------------------------------------------------
template <class R, class A1, class A2, class A3>
class Thread3: public Pointer< ArgumentThread3<R, A1, A2, A3> >
{
  void _init(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  void _create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  void _start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  public:
  void create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  void start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  friend Thread3<R, A1, A2, A3> 
    Create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  friend Thread3<R, A1, A2, A3> 
    Start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  Thread3(void);
  Thread3(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3);
  friend R Wait(const Thread3<R, A1, A2, A3> &t);
  operator Thread<R>();
};

// ----------------------------------------------------------------------------
//
// thread pointer with four arguments
//
// ----------------------------------------------------------------------------
template <class R, class A1, class A2, class A3, class A4>
class Thread4: public Pointer< ArgumentThread4<R, A1, A2, A3, A4> >
{
  void _init(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  void _create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  void _start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  public:
  void create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  void start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  friend Thread4<R, A1, A2, A3, A4>
    Create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  friend Thread4<R, A1, A2, A3, A4>
    Start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  Thread4(void);
  Thread4(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4);
  friend R Wait(const Thread4<R, A1, A2, A3, A4> &t);
  operator Thread<R>();
};

#endif
// ----------------------------------------------------------------------------
// end of rt_tptr.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
