// ----------------------------------------------------------------------------
//
// rt_stat.cc
// RT++ statistics
//
// $Id: rt_stat.cc,v 1.16 1996/04/04 07:06:50 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ---------------------------------------------------------------------------

#include "rt++.h"
 
RT_Argument rt_argument;
RT_Result *rt_result;
Statistics *rt_stat;

// ----------------------------------------------------------------------------
//
// initialize statistics
//
// ----------------------------------------------------------------------------
void stat_init(int verbose)
{
  rt_stat = (Statistics*)shm_alloc(sizeof(Statistics));
  rt_stat->Statistics(verbose);
}

// ----------------------------------------------------------------------------
//
// initialize statistics
// flush output streams before new processes are created
//
// ----------------------------------------------------------------------------
Statistics::Statistics(int v)
{
  verbose = v;
  gc_number = 0;
  gc_incs = 0;
  gc_blocks = 0;
  gc_bytes = 0;
  gc_time = 0;
  if (verbose)
    {
      cout << "------------------------------------------------------------------------------\n";
      cout << "RT++ " << RT_VERSION << " (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>\n";
      cout << "proc: " << sys->procnum();
      cout << ", mem: "  << shm_size();
      cout << ", heap: " << heap->heapsize();
      cout << ", blocks: " << heap->blocknumber();
      cout << ", stack: " << sys->stacksize();
      cout << "\n------------------------------------------------------------------------------\n";
      cout.flush();
      }
}

// ----------------------------------------------------------------------------
//
// start timers
//
// ----------------------------------------------------------------------------
void 
Statistics::start(void)
{
  start_time = real_time();
  start_clock = real_clock();
  start_all = real_all();
} 

// ----------------------------------------------------------------------------
//
// trigger garbage collection
//
// ----------------------------------------------------------------------------
void 
Statistics::gc_trigger(void)
{
  gc_wtime = real_time();
}

// ----------------------------------------------------------------------------
//
// start garbage collection
//
// ----------------------------------------------------------------------------
void 
Statistics::gc_start(void)
{
  gc_stime = real_time();
}

// ----------------------------------------------------------------------------
//
// too little heap space free
//
// ----------------------------------------------------------------------------
void 
Statistics::gc_free(int minimum)
{          
  if (verbose)
    {
      cout << "RT++ GC: less than " << minimum 
	<< " bytes heap (1 block per process) free.\n";
    }
}

// ----------------------------------------------------------------------------
//
// heap blocks too small
//
// ----------------------------------------------------------------------------
void
Statistics::gc_size(int minimum)
{
  if (verbose)
    {
      cout << "RT++ GC: all blocks smaller than " 
	<< minimum << " bytes (requested memory size).\n";
    }
}

// ----------------------------------------------------------------------------
//
// heap is extended by new cluster
//
// ----------------------------------------------------------------------------
void
Statistics::gc_inc(int size)
{
  gc_incs++;
  if (verbose)
    {
      cout << "RT++ GC: increase heap by " << size << " bytes.\n";
    }
}

// ----------------------------------------------------------------------------
//
// garbage collection
//
// ----------------------------------------------------------------------------
void 
Statistics::gc_end(int blocks, int bytes)
{
  int now = real_time();
  int wait = gc_stime-gc_wtime;
  int time = now-gc_stime;
  gc_number += 1;
  gc_blocks += blocks;
  gc_bytes += bytes;
  gc_time += time;
  gc_wait += wait;
  if (verbose)
    {
      cout << "RT++ GC "
	<< gc_number << " (" << ms_time(now-start_time) 
	  << " ms): " << blocks << " blocks with " << bytes << " bytes in " 
	    << ms_time(time) << " (+ " << ms_time(wait) << ") ms.\n";
    }
}

// ----------------------------------------------------------------------------
//
// report statistics
//
// ----------------------------------------------------------------------------
void 
Statistics::report(void)
{
  if (rt_result)
    {
      rt_result->init();
      if (verbose) rt_result->print();
    }
  else
    {
      RT_Result rt_res;
      rt_res.init();
      if (verbose) rt_res.print();
    }
}

// ----------------------------------------------------------------------------
//
// initialize result structure
//
// ----------------------------------------------------------------------------
void
RT_Result::init(void)
{
  ret_code = sys->ret_code();
  time_real = ms_time(real_time()-rt_stat->start_time);
  time_proc = ms_clock(real_clock()-rt_stat->start_clock);
  time_all = ms_all(real_all()-rt_stat->start_all);
  gc_number = rt_stat->gc_number;
  gc_inc = rt_stat->gc_incs;
  gc_wait = ms_time(rt_stat->gc_wait);
  gc_time = ms_time(rt_stat->gc_time);
  gc_blocks = rt_stat->gc_blocks;
  gc_bytes = rt_stat->gc_bytes;
}

// ----------------------------------------------------------------------------
//
// print result structure
//
// ----------------------------------------------------------------------------
void
RT_Result::print(void)
{
  cout << "------------------------------------------------------------------------------\n";
  cout << "RT++ execution\n";
  cout << "\tReturn code:\t" << ret_code << "\n";
  cout << "\tReal time(ms):\t" << time_real << "\n";
  cout << "\tProc time(ms):\t" << time_proc << "\n";
  cout << "\tAll procs(ms):\t" << time_all << "\n";
  cout << "RT++ garbage collections\n";
  cout << "\tNumber:\t\t"  << gc_number << "\n";
  cout << "\tIncrements:\t" << gc_inc << "\n";
  cout << "\tWaiting(ms):\t" << gc_wait << "\n";
  cout << "\tCollecting(ms):\t" << gc_time << "\n";
  cout << "\tBlocks:\t\t"  << gc_blocks << "\n";
  cout << "\tBytes:\t\t"   << gc_bytes  << "\n";
  cout << "------------------------------------------------------------------------------\n";
}

// ----------------------------------------------------------------------------
// end of rt_stat.cc
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
