// ----------------------------------------------------------------------------
//
// rt_thread.cc
// RT++ interface to system threads
//
// $Id: rt_thread.cc,v 1.15 1996/04/02 08:45:34 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------

#include "rt++.h"

// ----------------------------------------------------------------------------
//
// quick threads dependent functions
//
// ----------------------------------------------------------------------------
#ifdef RT_QT

// ----------------------------------------------------------------------------
//
// save stack pointer of thread as new thread pointer
//
// ----------------------------------------------------------------------------
inline void 
SystemThread::save(qt_t *sptr)
{
  thread = sptr;
}

// ----------------------------------------------------------------------------
//
// switch old thread to new thread
// caller is in charge of locking processor!!!
//
// ----------------------------------------------------------------------------
void*
switch_basic(qt_t *sptr, void* newthread, void *)
{
  CoreThread *oldthread = processor.get_sched();   // get scheduler thread
  oldthread->save(sptr);                           // save stack pointer
  processor.set_thread((CoreThread*)newthread);    // set new thread
  processor.unlock();                              // exit kernel mode
  return(0);                                       // satisfy type checker
}

// ----------------------------------------------------------------------------
//
// switch old thread to new thread where old thread is blocked on other thread
// blocker thread needs to be unlocked
// caller is in charge of locking processor!!!
//
// ----------------------------------------------------------------------------
void*
switch_block(qt_t *sptr, void* newthread, void *mutex)
{
  CoreThread *oldthread = processor.get_thread();   // get current thread
  oldthread->save(sptr);                            // save stack pointer
  processor.set_thread((CoreThread*)newthread);     // set new thread
  ((Mutex*)mutex)->unlock();                        // unlock mutex
  if (!processor.in_scheduler())                    // switching to scheduler
    {
      processor.unlock();                           // exit kernel mode
    }
  return(0);                                        // satisfy type checker
}

// ----------------------------------------------------------------------------
//
// switch old thread to new thread
// old thread is to be placed in ready queue
// caller is in charge of locking processor!!!
//
// ----------------------------------------------------------------------------
void*
switch_ready(qt_t *sptr, void* newthread, void *)
{
  CoreThread *oldthread = processor.get_thread();   // get current thread
  oldthread->save(sptr);                            // save stack pointer
  processor.set_thread((CoreThread*)newthread);     // set new thread
  sys->is_ready(oldthread);                         // place in ready queue
  if (!processor.in_scheduler())                    // switching to scheduler
    {
      processor.unlock();                           // exit kernel mode
    }
  return(0);                                        // satisfy type checker
}

// ----------------------------------------------------------------------------
//
// switch old thread to new thread
// old thread is terminated
// caller is in charge of locking processor!!!
//
// ----------------------------------------------------------------------------
void*
switch_term(qt_t *, void* newthread, void *)
{
  CoreThread *oldthread = processor.get_thread();   // get current thread
  oldthread->destroy();                             // free stack
  processor.set_thread((CoreThread*)newthread);     // set new thread
  if (!processor.in_scheduler())                    // switching to scheduler
    {
      processor.unlock();                           // exit kernel mode
    }
  return(0);                                        // satisfy type checker
}

#endif
// ----------------------------------------------------------------------------
// end of quick threads dependent functions
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// end of rt_thread.cc
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
